import { useEffect, useRef } from 'react'

// 防抖
const useDebounce = (
  fn: () => void,
  ms = 30,
  deps = [],
) => {
  let timeout = useRef<number | NodeJS.Timer>(0)

  useEffect(() => {
    if (timeout.current) clearTimeout(timeout.current)
    timeout.current = setTimeout(() => {
      fn()
    }, ms)
  }, deps)

  const cancel = () => {
    clearTimeout(timeout.current)
    timeout.current = 0
  }

  return [cancel]
}

export default useDebounce

/* DEMO
import { useDebounce } from 'hooks'
const Home = (props) => {
  const [a, setA] = useState(0)
  const [b, setB] = useState(0)
  const [cancel] = useDebounce(() => {
    setB(a)
  }, 2000, [a])

  const changeIpt = (e) => {
    setA(e.target.value)
  }
  return <div>
    <input type="text" onChange={changeIpt} />
    { b } { a }
  </div>
}
*/
