import { useEffect, useRef, useCallback } from 'react'

import { isNumber } from './utils'

function useInterval (
  fn: () => void,
  delay: number | undefined,
  options?: {
    immediate?: boolean;
  },
) {
  const immediate = options?.immediate

  const fnRef = useRef(fn)
  const timerRef = useRef<number | NodeJS.Timer>(0)

  useEffect(() => {
    if (!isNumber(delay) || delay < 0) return

    if (immediate) {
      fnRef.current()
    }

    timerRef.current = setInterval(() => {
      fnRef.current()
    }, delay)

    return () => {
      if (timerRef.current) {
        clearInterval(timerRef.current as NodeJS.Timer)
        timerRef.current = 0
      }
    }
  }, [delay])

  const clear = useCallback(() => {
    if (timerRef.current) {
      clearInterval(timerRef.current as NodeJS.Timer)
    }
  }, [])

  return clear
}

export default useInterval
