import { useEffect, useRef, useCallback } from 'react'

function useTimeout (fn: () => void, delay: number | undefined) {
  const fnRef = useRef(fn)
  const timerRef = useRef<number | NodeJS.Timer>(0)

  useEffect(() => {
    timerRef.current = setTimeout(() => {
      fnRef.current()
    }, delay)

    return () => {
      if (timerRef.current) {
        clearTimeout(timerRef.current as NodeJS.Timer)
        timerRef.current = 0
      }
    }
  }, [delay])

  const clear = useCallback(() => {
    if (timerRef.current) {
      clearTimeout(timerRef.current as NodeJS.Timer)
      timerRef.current = 0
    }
  }, [])

  return clear
}

export default useTimeout
